@@ -70,6 +70,7 @@ namespace Rstats {
   SV* pl_av_pop(SV*);
   void pl_av_unshift(AV*, SV*);
   void pl_av_unshift(SV*, SV*);
+  SV* pl_sv_bless(SV*, const char*);
   template <class X> X pl_to_c_obj(SV* perl_obj_ref) {
     SV* perl_obj = SvROK(perl_obj_ref) ? SvRV(perl_obj_ref) : perl_obj_ref;
     IV obj_addr = SvIV(perl_obj);
@@ -88,7 +89,7 @@ namespace Rstats {
   }
   IV pl_pregexec(SV*, REGEXP*);
 
-  namespace VectorType {
+  namespace Type {
     enum Enum {
       LOGICAL = 0,
       INTEGER = 1 ,
@@ -99,7 +100,6 @@ namespace Rstats {
   }
 
   namespace Util {
-    SV* args(SV*, SV*, SV*);
     IV is_perl_number(SV*);
     SV* cross_product(SV*);
     SV* pos_to_index(SV*, SV*);
@@ -303,7 +303,7 @@ namespace Rstats {
   }
 
   struct Vector {
-    Rstats::VectorType::Enum type;
+    Rstats::Type::Enum type;
     std::map<IV, IV>* na_positions;
     void* values;
   };
@@ -319,25 +319,25 @@ namespace Rstats {
     Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2 = Rstats::VectorFunc::new_logical(length); \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::CHARACTER : \
+        case Rstats::Type::CHARACTER : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_character_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
           } \
@@ -358,22 +358,22 @@ namespace Rstats {
     Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_complex_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_integer(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
@@ -391,22 +391,22 @@ namespace Rstats {
     Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_complex_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
@@ -424,22 +424,22 @@ namespace Rstats {
     Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
@@ -463,19 +463,19 @@ namespace Rstats {
       } \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v3 = Rstats::VectorFunc::new_logical(length); \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::CHARACTER : \
+        case Rstats::Type::CHARACTER : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_character_value(v1, i), Rstats::VectorFunc::get_character_value(v2, i)) ? 1 : 0); \
           } \
           break; \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i)) ? 1 : 0); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           for (IV i = 0; i < length; i++) { \
             try {\
               Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i)) ? 1 : 0); \
@@ -484,8 +484,8 @@ namespace Rstats {
             }\
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           for (IV i = 0; i < length; i++) { \
             try {\
               Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i), Rstats::VectorFunc::get_integer_value(v2, i)) ? 1 : 0); \
@@ -513,22 +513,22 @@ namespace Rstats {
       } \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v3; \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           v3 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_complex_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           v3 = Rstats::VectorFunc::new_integer(length); \
           for (IV i = 0; i < length; i++) { \
             try {\
@@ -557,22 +557,22 @@ namespace Rstats {
       } \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v3; \
-      Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
+      Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1); \
       switch (type) { \
-        case Rstats::VectorType::COMPLEX : \
+        case Rstats::Type::COMPLEX : \
           v3 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_complex_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i))); \
           } \
           break; \
-        case Rstats::VectorType::DOUBLE : \
+        case Rstats::Type::DOUBLE : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i))); \
           } \
           break; \
-        case Rstats::VectorType::INTEGER : \
-        case Rstats::VectorType::LOGICAL : \
+        case Rstats::Type::INTEGER : \
+        case Rstats::Type::LOGICAL : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
             Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i), Rstats::VectorFunc::get_integer_value(v2, i))); \
@@ -666,7 +666,7 @@ namespace Rstats {
     std::vector<NV>* get_double_values(Rstats::Vector*);
     std::vector<IV>* get_integer_values(Rstats::Vector*);
     
-    Rstats::VectorType::Enum get_type(Rstats::Vector*);
+    Rstats::Type::Enum get_type(Rstats::Vector*);
     void add_na_position(Rstats::Vector*, IV);
     bool exists_na_position(Rstats::Vector*, IV position);
     void merge_na_positions(Rstats::Vector*, Rstats::Vector*);
@@ -770,14 +770,14 @@ namespace Rstats {
     Rstats::Vector* clone(Rstats::Vector*);
   }
   
+  // Rstats::Func
   namespace Func {
     SV* new_array(SV*);
     void set_vector(SV*, SV*, Rstats::Vector*);
     Rstats::Vector* get_vector(SV*, SV*);
-    void set_dim_vector(SV*, SV*, Rstats::Vector*);
-    Rstats::Vector* get_dim_vector(SV*, SV*);
     SV* c(SV*, SV*);
     SV* to_c(SV*, SV*);
+
     SV* new_null(SV*); /* NULL */
     SV* new_na(SV*); /* NA */
     SV* new_nan(SV*); /* NaN */
@@ -789,17 +789,69 @@ namespace Rstats {
     SV* new_complex(SV*, SV*);
     SV* new_integer(SV*, SV*);
     SV* new_logical(SV*, SV*);
+
     SV* pi(SV*);
     SV* is_vector(SV*, SV*);
     SV* length_value(SV*, SV*);
     SV* values(SV*, SV*);
-    SV* set_dim(SV*, SV*, SV*);
-    SV* get_dim(SV*, SV*);
     SV* is_matrix(SV*, SV*);
     SV* is_array(SV*, SV*);
     SV* is_numeric(SV*, SV*);
     SV* type(SV*, SV*);
     IV to_bool(SV*, SV*);
+    SV* is_double(SV*, SV*);
+    SV* is_integer(SV*, SV*);
+    SV* is_complex(SV*, SV*);
+    SV* is_character(SV*, SV*);
+    SV* is_logical(SV*, SV*);
+    SV* is_data_frame(SV*, SV*);
+    SV* is_list(SV*, SV*);
+    SV* as_vector(SV*, SV*);
+    SV* new_data_frame(SV*);
+    SV* new_list(SV*);
+    SV* new_vector(SV*, SV*, SV*);
+    SV* copy_attrs_to(SV*, SV*, SV*);
+    SV* copy_attrs_to(SV*, SV*, SV*, SV*);
+
+    SV* as_integer(SV*, SV*);
+    SV* as_logical(SV*, SV*);
+    SV* as_complex(SV*, SV*);
+    SV* as_double(SV*, SV*);
+    SV* as_numeric(SV*, SV*);
+    SV* as_character(SV*, SV*);
+    SV* as(SV*, SV*, SV*);
+
+    SV* is_finite(SV*, SV*);
+    SV* is_infinite(SV*, SV*);
+    SV* is_nan(SV*, SV*);
+    SV* is_na(SV*, SV*);
+    SV* is_factor(SV*, SV*);
+    SV* is_ordered(SV*, SV*);
+    SV* clone(SV*, SV*);
+    SV* dim_as_array(SV*, SV*);
+    SV* decompose(SV*, SV*);
+    SV* compose(SV*, SV*, SV*);
+    SV* array(SV*, SV*);
+    SV* array(SV*, SV*, SV*);
+    SV* array_with_opt(SV*, SV*);
+    SV* args_h(SV*, SV*, SV*);
+    SV* as_array(SV*, SV*);
+    // class
+    SV* Class(SV*, SV*);
+    SV* Class(SV*, SV*, SV*);
+    SV* levels(SV*, SV*);
+    SV* levels(SV*, SV*, SV*);
+    SV* mode(SV*, SV*);
+    SV* mode(SV*, SV*, SV*);
+    
+    // dim
+    SV* dim(SV*, SV*, SV*);
+    SV* dim(SV*, SV*);
+    
+    SV* length(SV*, SV*);
+    SV* names(SV*, SV*, SV*);
+    SV* names(SV*, SV*);
+    
   }
 }
 
